<?php
/**
 * Class Welcome_Notice.
 */

defined( 'ABSPATH' ) || exit;

// phpcs:disable WordPress.WP.I18n.NonSingularStringLiteralDomain

/**
 * Class Welcome_Notice.
 */
class Welcome_Notice {

	/**
	 * @var $text_domain
	 */
	private static $text_domain;

	/**
	 * @var $message
	 */
	private static $message;

	/**
	 * @var $styles
	 */
	private static $styles;

	/**
	 * @var $plugins_list
	 */
	private static $plugins_list;

	/**
	 * @var $notice_action_text
	 */
	private static $notice_action_text;

	/**
	 * Init.
	 *
	 * Usage:
	 * ```
	 * Welcome_Notice::init( array(
	 *    'text_domain'        => 'default', // Required.
	 *    'message'            => 'Notice message can accept HTML.',
	 *    'plugins_list'       => array(
	 *        'plugin-slug' => array(
	 *            'name' => 'Plugin name',
	 *            'slug' => 'Plugin basename',
	 *        ),
	 *    ),
	 *    'styles'             => 'CSS styles .notice{ background:red }',
	 *    'notice_action_text' => 'Get Started (Notice action button text.)',
	 * ) )
	 * ```
	 *
	 * @param array $args Args.
	 * @return void
	 */
	public static function init( array $args ) {
		$defaults = array(
			'text_domain'        => 'default',
			'message'            => '',
			'plugins_list'       => array(
				'quick-demo-import' => array(
					'name' => 'Quick Demo Import',
					'slug' => 'quick-demo-import/qdi.php',
				),
			),
			'styles'             => '',
			'notice_action_text' => 'Get Started',
		);

		$args = self::recursive_parse_args( $args, $defaults );

		self::$text_domain        = $args['text_domain'];
		self::$message            = $args['message'];
		self::$plugins_list       = $args['plugins_list'];
		self::$styles             = $args['styles'];
		self::$notice_action_text = $args['notice_action_text'];

		add_action( 'admin_head', array( __CLASS__, 'print_notice_scripts' ) );
		add_action( 'admin_notices', array( __CLASS__, 'notice' ) );
		add_action( 'wp_ajax_' . self::$text_domain . '_dismiss_notice', array( __CLASS__, 'dismiss_notice' ) );
		add_action( 'wp_ajax_' . self::$text_domain . '_install_plugins', array( __CLASS__, 'install_plugins' ) );
	}

	/**
	 * Notice markup.
	 *
	 * @return void
	 */
	public static function notice() {
		if ( ! self::maybe_show_notice() ) {
			return;
		}
		?>
		<div class="notice notice-info <?php echo esc_attr( self::$text_domain ); ?>-notice">
			<div class="notice-content">
				<?php echo wp_kses_post( self::$message ); ?>
			</div>
			<div class="notice-action">
				<button class="button button-primary get-started">
					<?php
						printf(
						/* translators: %s: Action text. */
							esc_html__( '%s', self::$text_domain ), // phpcs:ignore WordPress.WP.I18n.NoEmptyStrings
							esc_html( self::$notice_action_text )
						);
					?>
				</button>
				<em class="plugin-install-info">
					<?php
						$plugin_names = array();
					foreach ( self::$plugins_list as $plugin ) {
						$plugin_names[] = $plugin['name'];
					}
						printf(
						/* Translators: %s: Plugin names. */
							esc_html__( 'Clicking the button will install and activate the %s.', self::$text_domain ),
							esc_html( implode( ', ', $plugin_names ) )
						);
					?>
				</em>
			</div>
			<button class="notice-dismiss">
				<span class="screen-reader-text"><?php esc_html_e( 'Dismiss this notice.', self::$text_domain ); ?></span>
			</button>
		</div>
		<?php
	}

	/**
	 * Maybe show notice.
	 *
	 * @return bool
	 */
	private static function maybe_show_notice() {
		return ! get_option( self::$text_domain . '_welcome_notice_dismissed' );
	}

	/**
	 * Print notice scripts.
	 *
	 * @return void
	 */
	public static function print_notice_scripts() {
		?>
		<style>
			.<?php echo esc_html( self::$text_domain ); ?>-notice {
				padding-right: 38px;
				position: relative;
			}
			.<?php echo esc_html( self::$text_domain ); ?>-notice .notice-content {
				padding-top: 10px;
				padding-bottom: 10px;
			}
			.<?php echo esc_html( self::$text_domain ); ?>-notice .notice-action {
				display: flex;
				align-items: center;
				margin-bottom: 10px;
			}
			.<?php echo esc_html( self::$text_domain ); ?>-notice .notice-action em {
				margin-left: 5px;
			}
			<?php self::validate_styles( self::$styles ) && print( self::$styles ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?>
		</style>
		<script type="text/javascript">
			( function() {
				if ( typeof jQuery === 'undefined' ) {
					return;
				}
				jQuery( document ).ready( function( $ ) {
					var nonces = {
							'dismiss': '<?php echo esc_js( wp_create_nonce( '_dismiss_nonce' ) ); ?>',
							'install': '<?php echo esc_js( wp_create_nonce( '_install_nonce' ) ); ?>'
						},
						prefix = '<?php echo esc_js( self::$text_domain ); ?>',
						plugins = <?php echo wp_json_encode( self::$plugins_list ); ?>,
						notice = $( '.' + prefix + '-notice' ),
						dismissBtn,
						getStartedBtn;

					if ( 0 === notice.length ) {
						return;
					}

					dismissBtn = notice.find( '.notice-dismiss' );
					getStartedBtn = notice.find( '.get-started' );

					dismissBtn.on( 'click', function( e ) {
						e.preventDefault();
						$.ajax( {
							type: 'POST',
							url: ajaxurl,
							data: {
								action: prefix + '_dismiss_notice',
								security: nonces.dismiss
							}
						} );
						notice.remove();
					} );

					getStartedBtn.on( 'click', function(e) {
						e.preventDefault();
						$( this ).addClass( 'updating-message' ).text( 'Processing..' );

						var deferred = $.Deferred().resolve();

						function ajax( args ) {
							return $.ajax( args );
						}

						$.each( plugins, function( key ) {
							var args = {
								type: 'POST',
								url: ajaxurl,
								data: {
									action: prefix + '_install_plugins',
									security: nonces.install,
									slug: key,
									plugin: plugins[key]['slug']
								}
							};
							deferred = deferred.then( function() {
								return ajax( args );
							} );
						} );

						deferred.then( function() {
							window.location.href = '<?php echo esc_url( admin_url( '/themes.php?page=demo-importer&browse=all' ) ); ?>'
						} );
					} )
				} );
			} )();
		</script>
		<?php
	}

	/**
	 * Validate styles.
	 *
	 * @param string $styles CSS styles.
	 * @return bool
	 */
	private static function validate_styles( $styles ) {
		return ! preg_match( '#</?\w+#', $styles );
	}

	/**
	 * Ajax callback for dismissing notice.
	 *
	 * @return void
	 */
	public function dismiss_notice() {
		check_ajax_referer( '_dismiss_nonce', 'security' );
		update_option( self::$text_domain . '_welcome_notice_dismissed', 1 );
		wp_send_json_success();
	}

	/**
	 * Install and activate Quick Demo Import plugin.
	 *
	 * @return void
	 */
	public static function install_plugins() {
		check_ajax_referer( '_install_nonce', 'security' );
		update_option( self::$text_domain . '_welcome_notice_dismissed', 1 );

		$slug   = sanitize_key( wp_unslash( $_POST['slug'] ) );
		$plugin = plugin_basename( sanitize_text_field( wp_unslash( $_POST['plugin'] ) ) );

		$state    = '';
		$response = array();

		if ( is_plugin_active( $slug ) ) {
			$state = 'activated';
		} elseif ( file_exists( WP_PLUGIN_DIR . "/$slug" ) ) {
			$state = 'installed';
		}

		if ( 'activated' === $state ) {
			wp_send_json_success();
		} elseif ( 'installed' === $state ) {
			if ( current_user_can( 'activate_plugin' ) ) {
				$result = activate_plugin( $plugin );

				if ( is_wp_error( $result ) ) {
					$response['errorCode']    = $result->get_error_code();
					$response['errorMessage'] = $result->get_error_message();
				}
			}
		} else {
			wp_enqueue_style( 'plugin-install' );
			wp_enqueue_script( 'plugin-install' );

			include_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
			include_once ABSPATH . 'wp-admin/includes/plugin-install.php';

			$api = plugins_api(
				'plugin_information',
				array(
					'slug'   => $slug,
					'fields' => array(
						'sections' => false,
					),
				)
			);

			$skin     = new WP_Ajax_Upgrader_Skin();
			$upgrader = new Plugin_Upgrader( $skin );
			$result   = $upgrader->install( $api->download_link );

			if ( $result ) {
				$response['installed'] = 'succeed';
			} else {
				$response['installed'] = 'failed';
			}

			// Activate plugin.
			if ( current_user_can( 'activate_plugin' ) ) {
				$result = activate_plugin( $plugin );

				if ( is_wp_error( $result ) ) {
					$response['errorCode']    = $result->get_error_code();
					$response['errorMessage'] = $result->get_error_message();
				}
			}
		}
		wp_send_json( $response );
	}

	/**
	 * Parse args recursively.
	 *
	 * @param array $args Args.
	 * @param array $defaults Defaults args.
	 * @return array Parsed args.
	 */
	private static function recursive_parse_args( array $args, array $defaults ) {
		$new_args = $defaults;

		foreach ( $args as $key => $value ) {
			if ( is_array( $value ) && isset( $new_args[ $key ] ) ) {
				$new_args[ $key ] = self::recursive_parse_args( $value, $new_args[ $key ] );
			} else {
				$new_args[ $key ] = $value;
			}
		}

		return $new_args;
	}
}
